/* PDF dumper and other examples based on PDFlib PLOP
 *
 * Version for Android, derived from standard dumper.java, encrypt.java and sign.java
 *
 * $Id: plop_examples.java,v 1.1.2.1 2015/03/02 13:10:00 rjs Exp $
 */

package com.pdflib;

import java.io.*;
import android.app.Activity;
import android.os.Bundle;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;
import android.util.Log;

public class plop_examples extends Activity implements OnClickListener {
    TextView textResult;
    Button buttonGo;
    EditText editInput;
    private static final String TAG = "PLOP";

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main);

        // Find UI views
        textResult = (TextView) findViewById(R.id.textResult);
        buttonGo = (Button) findViewById(R.id.buttonGo);
        buttonGo.setOnClickListener(this);
    }

    static final String infilename = "PLOP-datasheet.pdf";
    static final String docoptlist = "requiredmode=minimum";
    static final String globaloptlist = "";

    static plop plop = null;

    /* This is where input files live. Adjust as necessary. */
    static final String datadir = "/data/data/com.pdflib";
    static final String searchpath = datadir;

    private static String dumper(String filename) {
        String result = "";
        String out = "";
        int count, pcosmode;
        int i, doc;
        boolean plainmetadata;
        String objtype;
        String optlist;

        try

        {
            plop = new plop();
            plop.set_option("logging={filename=/data/data/com.pdflib/trace.txt remove}");

            optlist = "searchpath={" + searchpath + "}";
            plop.set_option(optlist);

            plop.set_option(globaloptlist);

            if ((doc = plop.open_document(filename, docoptlist)) == -1) {
                throw new Exception("ERROR " + plop.get_errmsg());
            }

            /* --------- general information (always available) */

            pcosmode = (int) plop.pcos_get_number(doc, "pcosmode");

            result += "       File name: " + plop.pcos_get_string(doc, "filename") + "\n";

            result += "     PDF version: " + plop.pcos_get_string(doc, "pdfversionstring") + "\n";

            result += "      Encryption: " + plop.pcos_get_string(doc, "encrypt/description") + "\n";

            result += "       Master pw: " + ((plop.pcos_get_number(doc, "encrypt/master") != 0) ? "yes" : "no") + "\n";

            result += "         User pw: " +
                    ((plop.pcos_get_number(doc, "encrypt/user") != 0) ? "yes" : "no") + "\n";

            result += "    Text copying: " +
                    ((plop.pcos_get_number(doc, "encrypt/nocopy") != 0) ? "no" : "yes") + "\n";

            result += "      Linearized: " +
                    ((plop.pcos_get_number(doc, "linearized") != 0) ? "yes" : "no") + "\n";

            if (pcosmode == 0) {
                result +=
                        "Minimum mode: no more information available\n" + "\n";
                plop.delete();
                System.exit(0);
            }

            /* --------- more details (requires at least user password) */

            result += "    PDF/X status: " + plop.pcos_get_string(doc, "pdfx") + "\n";

            result += "    PDF/A status: " + plop.pcos_get_string(doc, "pdfa") + "\n";
            result += "   PDF/UA status: " + plop.pcos_get_string(doc, "pdfua") + "\n";
            result += "   PDF/VT status: " + plop.pcos_get_string(doc, "pdfvt") + "\n";

            boolean xfa_present =
                    !plop.pcos_get_string(doc, "type:/Root/AcroForm/XFA").equals("null");
            result += "        XFA data: " + (xfa_present ? "yes" : "no") + "\n";

            result += "      Tagged PDF: " +
                    ((plop.pcos_get_number(doc, "tagged") != 0) ? "yes" : "no") + "\n" + "\n";

                /* --------- analyze digital signatures */

            count = (int) plop.pcos_get_number(doc, "length:signaturefields");

            for (i = 0; i < count; i++) {
                String sigtype;
                String sigfields;

                sigfields = "signaturefields[" + i + "]/fullname";
                result += "  Sig. field #" + i + 1 + ": '"
                        + plop.pcos_get_string(doc, sigfields) + "'," + "\n";

                sigfields = "signaturefields[" + i + "]/visible";
                result += " "
                        + ((int) plop.pcos_get_number(doc, sigfields) != 0 ? "visible" : "invisible") + "\n";

                sigfields = "signaturefields[" + i + "]/sigtype";
                sigtype = plop.pcos_get_string(doc, sigfields);

                if (sigtype.equals("none")) {
                    result += ", unsigned\n" + "\n";
                    continue;
                }

                result += " " + sigtype + " signature" + "\n";

                if (sigtype.equals("certification")) {
                    sigfields = "signaturefields[" + i + "]/permissions";
                    result += " (" + plop.pcos_get_string(doc, sigfields) + " allowed)" + "\n";
                }

                sigfields = "signaturefields[" + i + "]/cades";
                if ((int) plop.pcos_get_number(doc, sigfields) != 0)
                    result += ", CAdES";

                result += "\n";
            }
            if (count > 0) {
                result += "No. of revisions: "
                        + (int) plop.pcos_get_number(doc, "revisions") + "\n\n";
            }

            result += "  Reader-enabled: "
                    + plop.pcos_get_string(doc, "usagerights") + "\n";

            result += "    No. of pages: " +
                    (int) plop.pcos_get_number(doc, "length:pages") + "\n";

            result += "     Page 1 size: width=" +
                    (int) plop.pcos_get_number(doc, "pages[0]/width") + ", height=" +
                    (int) plop.pcos_get_number(doc, "pages[0]/height") + "\n";

            count = (int) plop.pcos_get_number(doc, "length:fonts");
            result += "    No. of fonts: " + count + "\n";

            for (i = 0; i < count; i++) {
                String fonts;

                fonts = "fonts[" + i + "]/embedded";
                if (plop.pcos_get_number(doc, fonts) != 0)
                    result += "embedded ";
                else
                    result += "unembedded ";

                fonts = "fonts[" + i + "]/type";
                result += plop.pcos_get_string(doc, fonts) + " font ";
                fonts = "fonts[" + i + "]/name";
                result += plop.pcos_get_string(doc, fonts);
            }

            result += "\n";

            plainmetadata = plop.pcos_get_number(doc, "encrypt/plainmetadata") != 0;

            if (pcosmode == 1 && !plainmetadata &&
                    plop.pcos_get_number(doc, "encrypt/nocopy") != 0) {
                result += "Restricted mode: no more information available" + "\n";
                plop.delete();
                System.exit(0);
            }

            /* ----- document info keys and XMP metadata (requires master pw) */

            count = (int) plop.pcos_get_number(doc, "length:/Info");

            for (i = 0; i < count; i++) {
                String info;
                String key;
                int len;

                info = "type:/Info[" + i + "]";
                objtype = plop.pcos_get_string(doc, info);

                info = "/Info[" + i + "].key";
                key = plop.pcos_get_string(doc, info);
                len = 16 - key.length();
                while (len-- > 0) result += " ";

                result += key + ": ";

                /* Info entries can be stored as string or name objects */
                if (objtype.equals("name") || objtype.equals("string")) {
                    info = "/Info[" + i + "]";
                    result += "'" + plop.pcos_get_string(doc, info) + "'";
                } else {
                    info = "type:/Info[" + i + "]";
                    result += "(" + plop.pcos_get_string(doc, info) +
                            " object)";
                }
            }

            result += "\n" + "   XMP meta data: ";

            objtype = plop.pcos_get_string(doc, "type:/Root/Metadata");
            if (objtype.equals("stream")) {
                byte[] contents;
                int len;

                contents = plop.pcos_get_stream(doc, "", "/Root/Metadata");
                result += contents.length + " bytes ";
                result += "";
            } else {
                result += "not present";
            }

            plop.close_document(doc, "");
        } catch (PLOPException e) {
            result = "PLOP exception occurred in dumper sample:" +
                    "[" + e.get_errnum() + "] " + e.get_apiname() +
                    ": " + e.get_errmsg();
        } catch (Exception e) {
            result = "Exception: " + e.getMessage();
        } finally {
            if (plop != null) {
                plop.delete();
            }
            Log.d(TAG, result);
        }
        return result;
    }

    private static String encrypt(String filename) {
        plop plop = null;
        String result = "Encrypt: OK";

        try {
            final String in_password = "";
            String optlist;

            /* parameters for the output document */
            final String out_filename = datadir + "/PLOP-datasheet-encrypted.pdf";
            final String out_master = "DEMO";
            final String out_user = "demo";
            final String permissions = "";
            int doc;

            plop = new plop();
            plop.set_option("logging={filename=/data/data/com.pdflib/trace-encrypt.txt remove}");

            optlist = "searchpath={" + searchpath + "}";
            plop.set_option(optlist);

            /* open unprotected input file without any password */
            optlist = "password {" + in_password + "} ";
            if ((doc = plop.open_document(filename, optlist)) == -1) {
                throw new Exception("Error: " + plop.get_errmsg());
            }

            /* create the output file with a password */
            optlist = "masterpassword {" + out_master + "} userpassword {" +
                    out_user + "} permissions {" + permissions + "} " +     "input="+ doc;

            if (plop.create_document(out_filename, optlist) == -1) {
                throw new Exception("Error: " + plop.get_errmsg());
            }

            /* close input and output files */
            plop.close_document(doc, "");

        } catch (PLOPException e) {
            result = "PLOP exception occurred in encrypt sample:\n" +
                    "[" + e.get_errnum() + "] " + e.get_apiname() +
                    ": " + e.get_errmsg() + "\n";
        } catch (Exception e) {
            result = e.getMessage();
        } finally {
            /* delete the PLOP object */
            if (plop != null) plop.delete();
        }
        return(result);
    }

    private static String sign(String filename) {
        plop plop = null;
        String result = "Sign: OK";

        String sign_opts =
                "sigtype=cms " +     /* use sigtype=cades for PAdES signature */
                "ltv=try " +         /* use ltv=full to ensure LTV status */
                /* for use with PKCS11
                   "engine=pkcs#11 digitalid={filename=cryptoki.dll} " +
                */
                "engine=builtin digitalid={filename=demorsa2048.p12} password={demo} ";

        try {
            /* parameters for the input document */
            String optlist;
            String fieldoptlist = "";

            int doc;
            plop = new plop();
            plop.set_option("logging={filename=" + datadir + "/trace-sign.txt remove}");

            /* open unprotected input file without any password */
            optlist = "searchpath {" + searchpath + "} ";
            plop.set_option(optlist);
            if ((doc = plop.open_document(filename, "")) == -1) {
                throw new Exception("Error: " + plop.get_apiname()
                        + ": " + plop.get_errmsg());
            }
            optlist = sign_opts;

            if (plop.prepare_signature(optlist) == -1) {
                throw new Exception("Error: " + plop.get_apiname()
                        + ": " + plop.get_errmsg());
            }

            if (plop.create_document(datadir + "/out-" + filename, "input=" + doc) == -1) {
                throw new Exception("Error: " + plop.get_apiname()
                        + ": " + plop.get_errmsg());
            }

            /* close input and output files */
            plop.close_document(doc, "");
        } catch (PLOPException e) {
            result = "PLOP exception occurred in sign sample:\n" +
                    "[" + e.get_errnum() + "] " + e.get_apiname() +
                    ": " + e.get_errmsg() + "\n";
        } catch (Exception e) {
            result = e.getMessage();
        } finally {
            /* delete the PLOP object */
            if (plop != null) plop.delete();
        }
        return(result);
    }


    public void onClick(View view) {
        String result;
        result = encrypt(infilename) + "\n";
        textResult.setText(result);
        result += sign(infilename) + "\n";
        textResult.setText(result);
        result += dumper(infilename);
        textResult.setText(result);
    }
}
