/* $Id: dynamicsign.java,v 1.1 2016/04/27 15:00:40 rp Exp $
 *
 * PDFlib PLOP DS:
 * generate a dynamic visualization document in memory with PDFlib,
 * and digitally sign a second document, using the dynamic document
 * for signature visualization.
 */

import java.io.*;
import com.pdflib.PLOPException;
import com.pdflib.plop;
import com.pdflib.PDFlibException;
import com.pdflib.pdflib;
import java.text.DateFormat;
import java.util.Date;

public class dynamicsign
{
    public static void main (String argv[])
    {
	pdflib pdf = null;
	plop plop = null;

	try {
	    byte[] contents;
	    Date now = new Date();
	    DateFormat fulldate = DateFormat.getDateInstance(DateFormat.LONG);
	    DateFormat fulltime = DateFormat.getTimeInstance(DateFormat.LONG);

	    String optlist;
	    int doc, visdoc;

	    /* This is where input files live. Adjust as necessary. */
	    String searchpath = "../data";

	    final String out_filename = "hellosign.pdf";

	    /* Basic signature options */
	    String sign_opts =
		"ltv=try " +         /* use ltv=full to ensure LTV status */
		/* for use with PKCS11
		"engine=pkcs#11 digitalid={filename=cryptoki.dll} "
		*/
		"engine=builtin digitalid={filename=demo_signer_rsa_2048.p12} password={demo} ";
            if (argv.length < 2) {
		throw new Exception("usage: dynamicfile <infile> <outfilename>");
	    }

	    /* ----------- Step 1: generate the document in memory  with PDFlib ---------- */

	    pdf = new pdflib();

	    /* create a new PDF document in memory */
	    pdf.begin_document("", ""); 

	    pdf.set_info("Creator", "dynamicsign.java");
	    pdf.set_info("Author", "Rainer Ploeckl");
	    pdf.set_info("Title", "Dynamic signature visualization document");

	    pdf.begin_page_ext(300, 50, "");
	    /* Prepare dynamic text for the signature visualization */
	    pdf.fit_textline("Digitally signed " + fulldate.format(now) + " at " + fulltime.format(now), 10, 10,
		"fontsize=10 fontname=Helvetica  encoding=unicode fillcolor=red");
	    pdf.end_page_ext("");

	    pdf.end_document("");
	    contents = pdf.get_buffer();

	    /* --------------- Step 2: sign the document -------------- */

	    plop = new plop();

	    optlist = "searchpath {" + searchpath + "} ";
	    plop.set_option(optlist);

	    /* Create a virtual PVF file from the document data in memory */
            plop.create_pvf("/pvf/visdoc.pdf", contents, "");

            if ((doc = plop.open_document(argv[0], "")) == -1) {
		throw new Exception("Error: " + plop.get_errmsg());
	    }
            /* open visualization document from PVF file */
            if ((visdoc = plop.open_document("/pvf/visdoc.pdf", "")) == -1) {
		throw new Exception("Error: " + plop.get_errmsg());
	    }
	    /* Add field option for visualization to base signature options */
	    optlist = sign_opts + " field={visdoc=" +  visdoc + " rect={10 10 adapt adapt}} ";

	    if (plop.prepare_signature(optlist) == -1) {
	        throw new Exception("Error: " + plop.get_apiname()
                                        + ": " + plop.get_errmsg());
	    }
	    
	    /* sign the output file */
	    if (plop.create_document(argv[1], "input=" + doc) == -1) {
	        throw new Exception("Error: " + plop.get_apiname()
                                        + ": " + plop.get_errmsg());
	    }

	    /* close the output input */
	    plop.close_document(doc, "");

            /* close the visualization document */
            plop.close_document(visdoc, "");

	    /* delete PVF file containing input document */
	    if (plop.delete_pvf("/pvf/visdoc.pdf") == -1) {
		throw new Exception("Error: " + plop.get_errmsg());
	    }


        } catch (PLOPException e) {
            System.err.print("PLOP exception occurred in dynamicsign sample:\n");
            System.err.print("[" + e.get_errnum() + "] " + e.get_apiname() +
                            ": " + e.get_errmsg() + "\n");
        } catch (PDFlibException e) {
            System.err.print("PLOP exception occurred in dynamicsign sample:\n");
            System.err.print("[" + e.get_errnum() + "] " + e.get_apiname() +
                            ": " + e.get_errmsg() + "\n");
        } catch (Exception e) {
            System.err.println(e.getMessage());
	} finally {
	    /* delete the PLOP object */
	    if (plop != null) plop.delete();

	    /* delete the PDFlib object */
	    if (pdf != null) {
		try { 
		    pdf.delete();
		} catch (Exception e) {
		    System.exit(2);
		}
	    }
	}
    }
}
